package com.ylx.web.controller.massage;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ylx.common.core.domain.R;
import com.ylx.common.utils.StringUtils;
import com.ylx.massage.domain.TOrder;
import com.ylx.massage.enums.Enumproject;
import com.ylx.massage.enums.OrderStatusEnum;
import com.ylx.massage.service.TOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;


import javax.annotation.Resource;
import java.util.List;

/**
 * 订单表 前端控制器
 */
@RestController
@Api(tags = {"订单管理"})
@RequestMapping("api/order/v1")
public class TOrderController {
    @Resource
    private TOrderService orderService;


    /**
     * 添加申请
     *
     * @param order
     * @return
     */
    @ApiOperation("添加申请")
    @RequestMapping(value = "wx/add", method = RequestMethod.POST)
    public String add(@RequestBody TOrder order) {
        return orderService.addOrder(order);
    }

    /**
     * 支付订单
     *
     * @param order
     * @return
     */
    @ApiOperation("支付订单")
    @RequestMapping(value = "wx/pay", method = RequestMethod.POST)
    public Boolean pay(@RequestBody TOrder order) {
        return orderService.payOrder(order);
    }

    /**
     * 取消订单
     *
     * @param order
     * @return
     */
    @ApiOperation("取消订单")
    @RequestMapping(value = "wx/cancle", method = RequestMethod.POST)
    public Boolean cancle(@RequestBody TOrder order) {
        return orderService.cancle(order);
    }

    /**
     * 拒绝订单
     *
     * @param order
     * @return
     */
    @ApiOperation("拒绝订单")
    @RequestMapping(value = "wx/jujue", method = RequestMethod.POST)
    public Boolean jujue(@RequestBody TOrder order) {
        return orderService.jujue(order);
    }


    /**
     * 接受订单
     *
     * @param order
     * @return
     */
    @ApiOperation("接受订单")
    @RequestMapping(value = "wx/takingOrders", method = RequestMethod.POST)
    public R takingOrders(@RequestBody TOrder order) {
        orderService.takingOrders(order);
        return R.ok();
    }

    /**
     * 确认订单
     *
     * @param order
     * @return
     */
    @ApiOperation("确认订单")
    @RequestMapping(value = "wx/confirm", method = RequestMethod.POST)
    public Boolean confirm(@RequestBody TOrder order) {
        return orderService.confirm(order);
    }

    /**
     * 获取订单信息
     *
     * @param param
     * @return
     */
    @ApiOperation("获取订单信息")
    @RequestMapping(value = "wx/getOrder", method = RequestMethod.GET)
    public R getOrder(Page<TOrder> page, TOrder param) {
        Page<TOrder> all = orderService.getAll(page, param);
        return R.ok(all);
    }


    /**
     * 分页查询数据
     */
    @ApiOperation("分页查询数据")
    @RequestMapping(value = "/select", method = RequestMethod.GET)
    public R selectSp(Page<TOrder> page, TOrder order) {
        LambdaQueryWrapper<TOrder> tOrderLambdaQueryWrapper = new LambdaQueryWrapper<>();
        tOrderLambdaQueryWrapper.eq(StringUtils.isNotBlank(order.getcJsId()), TOrder::getcJsId, order.getcJsId()).
                eq(StringUtils.isNotBlank(order.getcOpenId()), TOrder::getcOpenId, order.getcOpenId()).
                eq(null != order.getnStatus(), TOrder::getnStatus, order.getnStatus());
        // 获取查询返回结果
        Page<TOrder> pageSelect = orderService.page(page, tOrderLambdaQueryWrapper);
        return R.ok(pageSelect);
    }

    @ApiOperation("更新数据")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public R update(@RequestBody TOrder borrow) {
        return R.ok(orderService.updateById(borrow));
    }


    @ApiOperation("删除数据")
    @RequestMapping(value = "/del", method = RequestMethod.POST)
    public R del(@RequestBody TOrder borrow) {
        return R.ok(orderService.removeById(borrow));
    }

    @ApiOperation("根据id查询")
    @RequestMapping(value = "/getByid", method = RequestMethod.POST)
    public R<TOrder> getByid(@RequestBody TOrder borrow) {
        return R.ok(orderService.getById(borrow.getcId()));
    }

    @ApiOperation("订单状态")
    @RequestMapping(value = "/getStatus", method = RequestMethod.GET)
    public R getStatus() {
        List<Enumproject> statusEnum = OrderStatusEnum.getStatusEnum();
        return R.ok(statusEnum);
    }
}
