package com.ylx.massage.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ylx.common.constant.MassageConstants;
import com.ylx.common.utils.uuid.IdUtils;
import com.ylx.massage.domain.*;
import com.ylx.massage.enums.JsStatusEnum;
import com.ylx.massage.enums.OrderStatusEnum;
import com.ylx.massage.mapper.TOrderMapper;
import com.ylx.massage.service.TJsService;
import com.ylx.massage.service.TOrderService;
import com.ylx.massage.service.TWxUserService;
import com.ylx.massage.service.TXiangmuService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.Objects;


/**
 * 订单表 服务实现类
 */
@Service
public class TOrderServiceImpl extends ServiceImpl<TOrderMapper, TOrder> implements TOrderService {

    @Resource
    private TOrderMapper orderMapper;

    @Resource
    private TWxUserService wxUserService;

    @Resource
    private TXiangmuService xiangmuService;

    @Resource
    private TJsService jsService;

//    @Resource
//    private TInfoService infoService;

    /**
     * 添加订单
     *
     * @param order
     */
    @Override
    public String addOrder(TOrder order) {
        String cId = IdUtils.fastSimpleUUID();
        order.setcId(cId);
        order.setnStatus(OrderStatusEnum.WAIT_PAY.getCode());
        order.setDtCreateTime(LocalDateTime.now());
        return save(order) ? cId : null;
    }

    /**
     * 支付订单
     *
     * @param order
     */
    @Override
    public Boolean payOrder(TOrder order) {
        // 根据orderid查询订单信息
        TOrder orderNew = getById(order.getcId());
        // 从对应账户扣款
        TWxUser user = wxUserService.getById(orderNew.getcOpenId());
        if (user.getdBalance().compareTo(orderNew.getdTotalMoney()) < MassageConstants.INTEGER_ZERO) {
            return false;
        } else {
            // 更新用户金额 及下单此时
            TWxUser paramUser = new TWxUser();
            paramUser.setcOpenid(user.getcOpenid());
            paramUser.setdBalance(user.getdBalance().subtract(orderNew.getdTotalMoney()));
            paramUser.setdMoney(user.getdMoney().add(orderNew.getdTotalMoney()));
            paramUser.setnNum(user.getnNum() + MassageConstants.INTEGER_ZERO);
            wxUserService.updateById(paramUser);
            // 更新项目数据
            JSONArray objects = orderNew.getcGoods();
            objects.forEach(item -> {
                UpdateWrapper<TXiangmu> wrapper = new UpdateWrapper<>();
                // 获取参数
                wrapper.lambda().eq(TXiangmu::getcId, ((JSONObject) item).getString("cId"));
                // 设置数量
                wrapper.setSql(" n_sale_number = n_sale_number + " + ((JSONObject) item).getInteger("number"));
                xiangmuService.update(wrapper);
            });
            TOrder orderParam = new TOrder();
            orderParam.setcId(orderNew.getcId());
            orderParam.setnStatus(OrderStatusEnum.WAIT_JD.getCode());
            return updateById(orderParam);
        }
    }

    /**
     * 拒绝订单
     *
     * @param order
     */
    @Override
    public Boolean jujue(TOrder order) {
        TOrder orderNew = getById(order.getcId());

        TWxUser user = wxUserService.getById(orderNew.getcOpenId());
        // 更新用户金额 及下单此时
        TWxUser paramUser = new TWxUser();
        paramUser.setcOpenid(user.getcOpenid());
        // 金额归还对应账户
        paramUser.setdBalance(user.getdBalance().add(orderNew.getdTotalMoney()));
        // 消费金额对应减少
        paramUser.setdMoney(user.getdMoney().subtract(orderNew.getdTotalMoney()));
        // 下单次数减一
        paramUser.setnNum(user.getnNum() - MassageConstants.INTEGER_ONE);
        wxUserService.updateById(paramUser);

        // 更新项目数据
        JSONArray objects = orderNew.getcGoods();
        objects.forEach(item -> {
            UpdateWrapper<TXiangmu> wrapper = new UpdateWrapper<>();
            // 获取参数
            wrapper.lambda().eq(TXiangmu::getcId, ((JSONObject) item).getString("cId"));
            // 设置数量
            wrapper.setSql(" n_sale_number = n_sale_number - " + ((JSONObject) item).getInteger("number"));
            xiangmuService.update(wrapper);
        });
        TOrder orderParam = new TOrder();
        orderParam.setcId(orderNew.getcId());
        orderParam.setnStatus(OrderStatusEnum.REFUSE.getCode());
        return updateById(orderParam);
    }

    /**
     * 确认服务完成
     *
     * @param order
     * @return
     */
    @Override
    public Boolean confirm(TOrder order) {

        // 获取订单信息
        TOrder orderNew = getById(order);
        // 更新技师信息
        TJs jsParam = new TJs();
        jsParam.setcOpenId(orderNew.getcJsId());
        jsParam.setnStatus(JsStatusEnum.JS_SERVICEABLE.getCode());
        // 更新技师状态
        jsService.updateById(jsParam);
        // 更新技师钱包金额
        TJs jsById = jsService.getById(orderNew.getcJsId());
        // 获取技师抽成
        BigDecimal multiply = orderNew.getdTotalMoney().multiply(new BigDecimal(100 - jsById.getnBili()));
        multiply = multiply.divide(new BigDecimal(100), MassageConstants.INTEGER_TWO, RoundingMode.HALF_UP);
        // 获取技师所对应的用户
        TWxUser jsUser = wxUserService.getById(orderNew.getcJsId());
        // 更新余额
        jsUser.setdBalance(jsUser.getdBalance().add(multiply));
        // 更新总钱数
        jsUser.setdAllMoney(jsUser.getdAllMoney().add(multiply));
        wxUserService.updateById(jsUser);

//        // 如果该技师有推荐人员
//        if(StringUtils.isNotBlank(jsUser.getcUpUser())){
//            // 获取技师上级对应的用户
//            TWxUser jsUp = wxUserService.getById(jsUser.getcUpUser());
//            // 获取基本信息
//            List<TInfo> list = infoService.list();
//            BigDecimal up = orderNew.getdTotalMoney().multiply(new BigDecimal(list.get(0).getnBili()));
//            up = up.divide(new BigDecimal(100),2, RoundingMode.HALF_UP);
//            // 更新余额
//            jsUp.setdBalance(jsUp.getdBalance().add(up));
//            // 更新总钱数
//            jsUp.setdAllMoney(jsUp.getdAllMoney().add(up));
//            wxUserService.updateById(jsUp);
//        }
        // 更新订单
        orderNew.setnStatus(OrderStatusEnum.WAIT_EVALUATE.getCode());
        return updateById(orderNew);
    }

    /**
     * 取消订单
     *
     * @param order
     * @return
     */
    @Override
    public Boolean cancle(TOrder order) {
        // 获取订单信息
        // 根据orderid查询订单信息
        TOrder orderNew = getById(order.getcId());
        //待接单
        if (Objects.equals(orderNew.getnStatus(), OrderStatusEnum.WAIT_JD.getCode())) {
            TWxUser user = wxUserService.getById(orderNew.getcOpenId());
            // 更新用户金额 及下单此时
            TWxUser paramUser = new TWxUser();
            paramUser.setcOpenid(user.getcOpenid());
            // 金额归还对应账户
            paramUser.setdBalance(user.getdBalance().add(orderNew.getdTotalMoney()));
            // 消费金额对应减少
            paramUser.setdMoney(user.getdMoney().subtract(orderNew.getdTotalMoney()));
            // 下单次数减一
            paramUser.setnNum(user.getnNum() - MassageConstants.INTEGER_ONE);
            wxUserService.updateById(paramUser);

            // 更新项目数据
            JSONArray objects = orderNew.getcGoods();
            objects.forEach(item -> {
                UpdateWrapper<TXiangmu> wrapper = new UpdateWrapper<>();
                // 获取参数
                wrapper.lambda().eq(TXiangmu::getcId, ((JSONObject) item).getString("cId"));
                // 设置数量
                wrapper.setSql(" n_sale_number = n_sale_number - " + ((JSONObject) item).getInteger("number"));
                xiangmuService.update(wrapper);
            });
            TOrder orderParam = new TOrder();
            orderParam.setcId(orderNew.getcId());
            orderParam.setnStatus(OrderStatusEnum.CANCEL.getCode());
            return updateById(orderParam);

        } else if (Objects.equals(orderNew.getnStatus(), OrderStatusEnum.WAIT_PAY.getCode())) {
            //待付款
            TOrder orderParam = new TOrder();
            orderParam.setcId(orderNew.getcId());
            orderParam.setnStatus(OrderStatusEnum.CANCEL.getCode());
            return updateById(orderParam);
        } else {
            return false;
        }


    }


    @Override
    public Page<TOrder> getAll(Page<TOrder> page, TOrder param) {
        return orderMapper.getAll(page, param);
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void takingOrders(TOrder order) {
        if (order == null || StringUtils.isBlank(order.getcId())) {
            throw new IllegalArgumentException("订单对象不能为空");
        }

        TOrder orderNew = this.getById(order);

        // 检查订单对应的技师是否存在
        TJs js = jsService.getById(orderNew.getcJsId());
        if (js == null) {
            throw new IllegalStateException("无法找到对应的技师");
        }

        // 更新技师状态
        js.setnStatus(JsStatusEnum.JS_SERVICE.getCode());
        // 确保js.getnNum()不为null，避免 NullPointerException
        int num = js.getnNum() == null ? 0 : js.getnNum();
        js.setnNum(num + MassageConstants.INTEGER_ONE);
        jsService.updateById(js);
        // 更新订单状态
        TOrder orderParam = new TOrder();
        orderParam.setcId(order.getcId());
        orderParam.setnStatus(OrderStatusEnum.SERVICE.getCode());
        this.updateById(orderParam);
    }
}
